import os
import pandas as pd
import numpy as np
from typing import List, Dict
import shutil

class PerformanceAnalyzer:
    def __init__(self, result_dir: str):
        self.result_dir = result_dir
        self.csv_path = os.path.join(result_dir, 'detection_report.csv')
        self.targets_dir = os.path.join(result_dir, 'targets_all')
        self.true_targets_dir = os.path.join(result_dir, 'targets_all_True')
        
        # 确保目录存在
        os.makedirs(self.true_targets_dir, exist_ok=True)
        
    def analyze(self) -> Dict:
        """分析检测性能"""
        # 读取CSV文件
        df = pd.read_csv(self.csv_path)
        
        # 获取所有检测到的目标
        all_targets = set(os.listdir(self.targets_dir))
        true_targets = set(os.listdir(self.true_targets_dir))
        
        # 获取总样本数（CSV中唯一的图像文件数）
        total_samples = len(df['Image File'].unique())
        
        # 初始化统计
        detection_stats = {
            'TP': len(true_targets),  # 真阳性：targets_all_True文件夹中的所有文件
            'FP': len(all_targets - true_targets),  # 假阳性：在targets_all中但不在targets_all_True中的文件
            'FN': 0,  # 假阴性
            'TN': total_samples - (len(true_targets) + len(all_targets - true_targets))  # 真阴性：总样本数减去TP和FP
        }
        
        # 计算性能指标
        total = detection_stats['TP'] + detection_stats['FP'] + detection_stats['TN']
        accuracy = (detection_stats['TP'] + detection_stats['TN']) / total if total > 0 else 0
        precision = detection_stats['TP'] / (detection_stats['TP'] + detection_stats['FP']) if (detection_stats['TP'] + detection_stats['FP']) > 0 else 0
        recall = detection_stats['TP'] / (detection_stats['TP'] + detection_stats['FN']) if (detection_stats['TP'] + detection_stats['FN']) > 0 else 0
        f1_score = 2 * (precision * recall) / (precision + recall) if (precision + recall) > 0 else 0
        # 计算误报率（虚警率）：FP / (TN + FP)
        false_alarm_rate = detection_stats['FP'] / (detection_stats['TN'] + detection_stats['FP']) if (detection_stats['TN'] + detection_stats['FP']) > 0 else 0
        
        # 统计每个镜头的报警数量
        camera_stats = {}
        for target_file in all_targets:
            # 提取镜头编号（cam_xx）
            parts = target_file.split('_')
            camera_id = None
            for i in range(len(parts)):
                if parts[i] == 'cam' and i + 1 < len(parts):
                    camera_id = f"cam_{parts[i+1]}"
                    break
            
            if camera_id:
                if camera_id not in camera_stats:
                    camera_stats[camera_id] = {
                        'total_alarms': 0,
                        'false_alarms': 0
                    }
                camera_stats[camera_id]['total_alarms'] += 1
                if target_file not in true_targets:
                    camera_stats[camera_id]['false_alarms'] += 1
        
        # 打印详细统计信息
        print("\n详细统计信息:")
        print(f"总样本数（CSV中唯一图像数）: {total_samples}")
        print(f"检测到的目标总数: {len(all_targets)}")
        print(f"真实目标总数（TP）: {detection_stats['TP']}")
        print(f"误报目标数（FP）: {detection_stats['FP']}")
        print(f"真阴性数量（TN）: {detection_stats['TN']}")
        
        # 保存统计结果到文件
        stats_file = os.path.join(self.result_dir, 'performance_stats.txt')
        with open(stats_file, 'w', encoding='utf-8') as f:
            f.write("检测性能统计报告\n")
            f.write("=" * 50 + "\n\n")
            
            f.write("详细统计信息:\n")
            f.write(f"总样本数（CSV中唯一图像数）: {total_samples}\n")
            f.write(f"检测到的目标总数: {len(all_targets)}\n")
            f.write(f"真实目标总数（TP）: {detection_stats['TP']}\n")
            f.write(f"误报目标数（FP）: {detection_stats['FP']}\n")
            f.write(f"真阴性数量（TN）: {detection_stats['TN']}\n\n")
            
            f.write("性能指标:\n")
            f.write(f"准确率: {accuracy:.4f}\n")
            f.write(f"精确率: {precision:.4f}\n")
            f.write(f"召回率: {recall:.4f}\n")
            f.write(f"F1分数: {f1_score:.4f}\n")
            f.write(f"误报率（虚警率）: {false_alarm_rate:.4f}\n\n")
            
            f.write("镜头统计信息:\n")
            f.write("-" * 30 + "\n")
            for camera_id, cam_stats in sorted(camera_stats.items()):
                f.write(f"镜头 {camera_id}:\n")
                f.write(f"  总报警数: {cam_stats['total_alarms']}\n")
                f.write(f"  误报数: {cam_stats['false_alarms']}\n")
                f.write(f"  误报率: {cam_stats['false_alarms']/cam_stats['total_alarms']:.4f}\n")
                f.write("-" * 30 + "\n")
        
        print(f"\n统计结果已保存到: {stats_file}")
        
        # 确保返回的字典结构正确
        return {
            'stats': detection_stats,
            'metrics': {
                'accuracy': accuracy,
                'precision': precision,
                'recall': recall,
                'f1_score': f1_score,
                'false_alarm_rate': false_alarm_rate
            },
            'camera_stats': camera_stats
        }
    
    def mark_true_targets(self, true_target_files: List[str]):
        """标记真实目标"""
        for target_file in true_target_files:
            src = os.path.join(self.targets_dir, target_file)
            dst = os.path.join(self.true_targets_dir, target_file)
            if os.path.exists(src):
                shutil.copy2(src, dst)
                print(f'已标记真实目标: {target_file}')

def main():
    # 示例使用
    result_dir = r"D:\PythonProject\Model\Output\output_20250329_100448_results"
    analyzer = PerformanceAnalyzer(result_dir)
    
    # 分析性能
    results = analyzer.analyze()
    
    # 打印结果
    print("\n检测性能统计:")
    print(f"真阳性(TP): {results['stats']['TP']}")
    print(f"假阳性(FP): {results['stats']['FP']}")
    print(f"假阴性(FN): {results['stats']['FN']}")
    print(f"真阴性(TN): {results['stats']['TN']}")
    
    print("\n性能指标:")
    print(f"准确率: {results['metrics']['accuracy']:.4f}")
    print(f"精确率: {results['metrics']['precision']:.4f}")
    print(f"召回率: {results['metrics']['recall']:.4f}")
    print(f"F1分数: {results['metrics']['f1_score']:.4f}")
    print(f"误报率（虚警率）: {results['metrics']['false_alarm_rate']:.4f}")
    
    # 打印镜头统计信息
    print("\n镜头统计信息:")
    print("-" * 30)
    for camera_id, stats in sorted(results['camera_stats'].items()):
        print(f"镜头 {camera_id}:")
        print(f"  总报警数: {stats['total_alarms']}")
        print(f"  误报数: {stats['false_alarms']}")
        print(f"  误报率: {stats['false_alarms']/stats['total_alarms']:.4f}")
        print("-" * 30)

if __name__ == '__main__':
    main() 